/*!
@mainpage
@copyright &copy; Rohde & Schwarz GmbH & Co.KG, Munich

@section gen 1 General

This file defines the interface for custom DLLs which can be used with the ZNX firmware. The mechanism for adding and removing DLLs is described in the manual
of the firmware. The DLL is installed in the firmware as *.zip file. This file must include the *.dll file with the functions defined in this header. 
Additionally this *.zip file is allowed to contain further files. These are stored with the DLL in an internal folder. The firmware will not access these files
directly. The files are kept until the package is removed again. Files with the extension *.json are used by the firmware to create customized buttons if
possible (see chapter 6).

@section fun 2 Functions in the Interface

The interface consists of a set of functions which must be implemented. They can be divided into four groups:
@subsection dll 2.1. DLL Handling: The functions in this group are
     - RsVnaOnLoadDll
     - RsVnaOnUnloadDll
     These functions are rarely called from the firmware. Typical calls are when starting or stopping the firmware.
@subsection gp 2.2. General Properties: The functions in this group are
     - RsVnaGetVersion
     - RsVnaGetNumberOfTaskTypes
     - RsVnaGetTaskTypes
     - RsVnaGetUnit
     These functions provide information about the capabilities of the DLL to the firmware. Calls can occur often e.g. to retrieve values for the
     visualization.
@subsection tr 2.3. Trace Calculation: The functions in this group are
     - RsVnaGetNumberOfHumanNames (Init)
     - RsVnaGetTraceRequirements (Init)
     - RsVnaCalculateTrace (Meas)
     These functions are used to actually provide data for a trace which shall be used by the firmware. Calls are explained in the next part.
@subsection dev 2.4. Device Control: The functions in this group are
     - RsVnaGetInterruptRequests (Init)
     - RsVnaInterruptCallback (Meas)
@subsection dum 2.5. Summary 
These functions are used to control external devices and provide the results to the firmware. Calls are explained in the next part.
All these functions have in common that the firmware expects that the same input results to the same output. The only exception is RsVnaInterruptCallback
(for details see function description).

@section gfs 3 General Firmware Structure

To understand the impact of the "Trace Calculation" (2.3) functions and the "Device Control" (2.4) functions a more detailed understanding of the processes in
the firmware is required. Three general parts exist:
@subsection gui 3.1. GUI/Parser/Internal Logic: 
     This part of the firmware directly interacts with the user. The task is to setup the measurements. This part of the firmware also contains the
     functionality which computes the trace from the complex data including math operations, statistics, etc.. In this part of the firmware, the functions
     from the "DLL Handling" (2.1) and "General Properties" (2.2) are called as needed by the firmware.
@subsection init 3.2. Initialization: 
     In this part of the firmware the required wave measurements are computed from the settings made in "GUI/Parser/Internal Logic" (3.1) and passed to the
     hardware for execution. This process is triggered when settings change. In this process the following steps occur in this order:
     - Initialize Trace Calculation (e.g. Format or Trace Math)
     - Initialize Averaging
     - Calls to functions from "Trace Calculation" (2.3) marked with "Init"
     - Preparation of the required wave measurements (e.g. decomposition of a ratio into two waves)
     - Initialization of the hardware during this step call to functions from "Device Control" (2.4) marked with "Init" occur.
@subsection meas 3.3. Measurement: 
     In this part of the firmware the data from the actual measurement is processed and further measurements are controlled. In continuous sweep mode this part
     of the firmware is permanently working to process the incoming measurements. The speed of this code can have a big impact on the device performance. 
     The following steps occur in this order:
     - Delivery of the measured waves. In this part also the functions from "Device Control" (2.4) marked with "Meas" are called to collect the measurements by
       additional devices.
     - Calculation of composite measurements (e.g. the division of two waves to get a ratio)
     - Calls to functions from "Trace Calculation" (2.3) marked with "Meas"
     - Calculation of averages
     - Application of trace calculations (e.g. Format or Trace Math)

@section con 4 Concept for Different Drive Ports

Another concept to have in mind when implementing this interface is different drive ports. The interface uses the "Number of Sweep Points" as it is used by the
user interface of the firmware. However, the actual steps required during the measurement might be higher due to the fact that one measurement is using a
different drive port (e.g. forward and backwards measurement). To associate a task with a drive port this drive port is part of the task type usually denoted 
"PX" or "DX". This is the same for tasks requested from the DLL. Especially for the control of external devices this can have a big impact.

@section mem 5 Memory Handling

For all functions in this interface the ZNX firmware allocates and deletes the required memory. The pointers provided already have the required memory. This
memory can be accessed and the values can be read or written according to the description of the function. 
- Do not delete the memory provided by the interface functions. This is done by the firmware.
- Expect that all the memory is deleted immediately after the function of this interface returns. Do not store any pointers.

@section cus 6 Customized Buttons in the Firmware

As part of the package installed with the DLL it is possible to add customized buttons to the firmware. This is done by *.json files. Simply add such a file to
the *.zip container with the following content:
@code { 
  "SoftKeyId": "E_Meas_Spectrum",            // Decides to which SoftTool the Button is added. You find theses Ids in “MMI_IDs” folder. ID greater than 10000.
  "PathOfExecuteable": "executable.exe",     // Path for the executable to run when the button is pressed.
  "Icon": "C:\\path\\to\\image.svg",         // Icon for the button. It is shown on the right side. Use a *.svg file. 
  "ButtonText": "DoSomething",               // Text on the button (for two lines use \n). ButtonText and Icon should not be too wide to fit in the SoftTool.
  "Separator": false,                        // A white line is shown above the button if true is set.
  "SeperatorText": "label",                  // The given text is shown between the Separator and the Button. The text should not be too wide for the SoftTool.
  "CmdLineOptions": ["String1",”String2”],   // Command Line parameters for the executable.
  "UserInteraction": false                   // Set the macro modal if “true”.
} @endcode
Optionally also a Menu.json can be added with the following content
@code{ 
  "SoftMenuId": "E_SoftKeyMenu_Display",     // Decides to which SoftToolMenu the Button is added. You find theses Ids in “MMI_IDs” folder (E_SoftKeyMenu_XXX). 
  "TabText": "Test"                          // Text on the Tab for the SoftTool.
}@endcode

@section exc 7 Exception settings

As the VNA Firmware translates structured exceptions into C++ Exception any structured exception usage inside your dll may be translated to an unspecified
c++ exception. This is subject to change and as a user you shouldn't rely on it.
*/


/**
@file      RsVnaCustomDllInterface.h
@copyright &copy; Rohde & Schwarz GmbH & Co.KG, Munich
*/

#ifndef RS_VNA_CUSTOM_DLL_INTERFACE_28F7221D_FAA2_450B_B88A_BB6BED9C0262
#define RS_VNA_CUSTOM_DLL_INTERFACE_28F7221D_FAA2_450B_B88A_BB6BED9C0262


#ifndef RS_VNA_API_IMPORTS
#define RS_VNA_API __declspec(dllexport)  ///< Usage as Dll export
#else
#define RS_VNA_API __declspec(dllimport)  ///< Usage as Dll import
#endif

/// Interface version in major release, minor release and patch level.
#define RS_VNA_DLL_INTERFACE_MAJOR_RELEASE 1 ///< Major Release Version of this interface
/// Interface version in major release, minor release and patch level.
#define RS_VNA_DLL_INTERFACE_MINOR_RELEASE 0 ///< Minor Release Version of this interface
/// Interface version in major release, minor release and patch level.
#define RS_VNA_DLL_INTERFACE_PATCH_LEVEL 0   ///< Path Level of this interface

/// Contains the maximal length for the here used char types.
#define RS_VNA_MAX_CHAR_SIZE 512  


#ifdef __cplusplus
extern "C" {
#endif

  /// Contains a version number in major release, minor release and patch level.
  struct RsVnaVersion
  {
    unsigned int majorRelease; ///< Major release Version
    unsigned int minorRelease; ///< Minor release Version
    unsigned int patchLevel;   ///< Patch Version
  };

  /// Contains an ID, consisting of the following four parameters, to identify a specific task.
  struct RsVnaTaskId
  {
    char dllFilename[RS_VNA_MAX_CHAR_SIZE];     ///< Contains the DLL filename (excluding the file extension).
    char additionalInput[RS_VNA_MAX_CHAR_SIZE]; ///< Contains the additional input text (e.g. "Ch1" for channel identification).
    unsigned int drivingPort;                   ///< Contains the drive port number (1-based). Zero if called for all drive ports.
    char taskType[RS_VNA_MAX_CHAR_SIZE];       ///< Contains the task type name.
  };

  /// Contains a complex value in real and imaginary part.
  struct RsVnaComplex
  {
    double realPart;             ///< real part of the complex number
    double imaginaryPart;        ///< imaginary part of the complex number
  };

  /// Contains an human name (same as in SCPI) with a corresponding complex value for a specific measurement point.
  struct RsVnaTraceData
  {
    char humanName[RS_VNA_MAX_CHAR_SIZE];  ///< Human name associated with the value
    struct RsVnaComplex value;             ///< Complex value provided for the humanName
  };

  /// Contains all available units for the task type.
  enum ERsVnaMeasUnit
  {
    RS_VNA_UNIT,      ///< No Unit
    RS_VNA_GRAD,      ///< Degree
    RS_VNA_SECONDS,   ///< Seconds
    RS_VNA_DECIBEL,   ///< dB
    RS_VNA_OHM,       ///< Ohm
    RS_VNA_SIEMENS,   ///< Siemens
    RS_VNA_VOLT,      ///< Volt
    RS_VNA_WATT,      ///< Watt
    RS_VNA_DBM,       ///< dBm
    RS_VNA_DB_MICRO_V,///< dB / &mu;V
    RS_VNA_SECONDS_HZ,///< s / Hz
    RS_VNA_DBC,       ///< dBc
    RS_VNA_DBM_HZ,    ///< dBm / Hz
    RS_VNA_DB_HZ,     ///< dB / Hz
    RS_VNA_HENRY,     ///< Henry
    RS_VNA_FARAD,     ///< Farad
    RS_VNA_AMPERE,    ///< Ampere
  };

  /// Contains the input type.
  enum ERsVnaInputType
  {
    RS_VNA_SAME, ///< If the input should be equal to the output, so no trace calculation is required (no call to method "RsVnaCalculateTrace()").
    RS_VNA_LIST  ///< If a list of human names (same as in SCPI) of a specific task type is returned, required for the trace calculation later.
  };

  /// Contains the custom DLL interrupt type, which describes how often an interrupt should be occurred.
  enum ERsVnaInterruptType
  {
    RS_VNA_CHANNEL,      ///< If the interrupt is requested per channel.
    RS_VNA_SEGMENT,      ///< If the interrupt is requested per segment.
    RS_VNA_DRIVING_PORT, ///< If the interrupt is requested per drive port.
    RS_VNA_MEAS_POINT,   ///< If the interrupt is requested per measurement point.
  };

  /// Contains the interrupt position, which describes where the interrupt should be occurred.
  enum ERsVnaInterruptPosition
  {
    RS_VNA_NONE, ///< If no interrupt is requested.
    RS_VNA_PRE,  ///< If the interrupt is requested before the measurement of the specified interrupt type.
    RS_VNA_POST, ///< If the interrupt is requested after the measurement of the specified interrupt type.
    RS_VNA_BOTH  ///< If the interrupt is requested before and after the measurement of the specified interrupt type.
  };

  /** Is called when loading the DLL.

  Calls to this function occur when starting the firmware or adding/removing DLLs.

  This function is optional.

  @return 0 if no error occurred, else an error code to identify the occurred error.
          A detailed error information is provided by the method "RsVnaGetErrorInfo()".
  */
  RS_VNA_API int RsVnaOnLoadDll();

  /** Is called when unloading the DLL.

  Calls to this function occur when stopping the firmware or adding/removing DLLs.

  This function is optional.

  @return 0 if no error occurred, else an error code to identify the occurred error.
          A detailed error information is provided by the method "RsVnaGetErrorInfo()".
  */
  RS_VNA_API int RsVnaOnUnloadDll();

  /** Gets the DLL version and the interface version in major release, minor release and patch level.

  This function provides information about the versions to the firmware. 
  - The DLL version is only informative. It is displayed in the GUI. Developers can use it to distinguish different versions of their DLL.
  - The interface version should be the version defined by RS_VNA_DLL_INTERFACE_XXX. It identifies the implemented interface to the firmware.
    Any other value returned might create undefined behavior and firmware crashes. 

  This function is mandatory and must be implemented.

  @param[in,out] dllVersion       Contains the internal version of the DLL.
  @param[in,out] interfaceVersion Contains the version of the RsVnaCustomDllInterface which was used to compile the DLL.
  @return        0 if no error occurred, else an error code to identify the occurred error.
                 A detailed error information is provided by the method "RsVnaGetErrorInfo()".
  */
  RS_VNA_API int RsVnaGetVersion(struct RsVnaVersion* dllVersion, struct RsVnaVersion* interfaceVersion);

  /** Gets the number of task type names.

  The DLL can define a number of task types identified by a name (see "RsVnaGetTaskTypes"). This function returns the total number of these types.
  The returned value is used as input parameter for "RsVnaGetTaskTypes".

  This function is mandatory and must be implemented.

  @param[in,out] numberOfTaskTypes Contains the number of task type names.
  @return        0 if no error occurred, else an error code to identify the occurred error.
                 A detailed error information is provided by the method "RsVnaGetErrorInfo()".
  */
  RS_VNA_API int RsVnaGetNumberOfTaskTypes(unsigned int* numberOfTaskTypes);

  /** Gets the task type names.

  This method defines the task types available by the DLL. A task to measure for the DLL is identified by the struct "RsVnaTaskId". The strings returned by 
  this function define the entries in RsVnaTaskId::taskType passed later to the DLL. The concept for the task type is similar to the concept of a task
  type wave, ratio or S-Parameter in the firmware. Also for external devices task types are used. These allow to visualize data even if it is not the main use
  case. In task types and additional default input the characters "*<>?\|/: are NOT allowed.

  This function is mandatory and must be implemented.

  @param[in]     numberOfTaskTypes      Contains the number of task type names as provided by the method "RsVnaGetNumberOfTaskTypes()".
  @param[in,out] taskTypesList          Contains a list of names for the task types the DLL can handle with units.
                                        These task type names are available in the UI and will be provided to the DLL to identify the selection of the user.
  @param[in,out] additionalDefaultInput Contains the default text for the additional input text field in the UI (e.g. "Ch1" for channel identification).
  @return        0 if no error occurred, else an error code to identify the occurred error.
                 A detailed error information is provided by the method "RsVnaGetErrorInfo()".
  */
  RS_VNA_API int RsVnaGetTaskTypes(const unsigned int numberOfTaskTypes, char** taskTypesList, char** additionalDefaultInput);

  /** Gets the corresponding unit for a specific task type.

  Associated with each task type is a unit. This unit is used by the firmware to interpret the values returned by "RsVnaCalculateTrace". The calculation of
  derived quantities depends on this unit.

  This function is optional, but it must be implemented if any Task shall be used as trace. In this case you also need the functions 
  "RsVnaGetNumberOfHumanNames", "RsVnaGetTraceRequirements" and "RsVnaCalculateTrace" are required.

  @param[in]     taskId Contains the task ID to identify the corresponding task type.
  @param[in,out] unit   Contains the corresponding unit for the passed task ID.
  @return        0 if no error occurred, else an error code to identify the occurred error.
                 A detailed error information is provided by the method "RsVnaGetErrorInfo()".
  */
  RS_VNA_API int RsVnaGetUnit(const struct RsVnaTaskId taskId, enum ERsVnaMeasUnit* unit);

  /** Gets the number of human names.

  For a given "RsVnaTaskId" a certain number of input traces can be requested from the firmware (e.g. if the DLL calculates the ratio of two waves this
  function should return "2" for the two waves). The human name is the same name used by the SCPI interface to request measurement. The value returned by this
  function is used as input for "RsVnaGetTraceRequirements".

  This function is optional, but if it is implemented, also the functions "RsVnaGetUnit", "RsVnaGetTraceRequirements" and "RsVnaCalculateTrace" are required.

  @param[in]     taskId             Contains the task ID to identify the corresponding task type.
  @param[in,out] numberOfHumanNames Contains the number of human names.
  @return        0 if no error occurred, else an error code to identify the occurred error.
                 A detailed error information is provided by the method "RsVnaGetErrorInfo()".
  */
  RS_VNA_API int RsVnaGetNumberOfHumanNames(const struct RsVnaTaskId taskId, unsigned int* numberOfHumanNames);

  /** Gets the trace requirements.
  
  This Function does the preparations for the calculation of a trace. The function is called for all traces used. THe trace is identified by the "RsVnaTaskId".
  The firmware must know which data it should be provided for this calculation (e.g. if the desired calculation is the ratio of two waves, these two waves 
  must be specified here). This includes data which might originate from the DLL provided by RsVnaInterruptCallback. The specification is done by the  
  "ERsVnaInputType" and a list of human names:
  - If no calculation is required the "ERsVnaInputType" SAME shall be used (e.g. if the trace shall display data provided by RsVnaInterruptCallback)
  - Otherwise "ERsVnaInputType" LIST must be selected to provide the names. These names are the same as used for the traces in the SCPI interface of the 
    firmware. Most trace names accepted via SCPI are also available here:
    - Not available are noise figure and 2D compression point measurements
    - Names from DLL packages are accepted. They are only forwarded to the "Device Control" (2.4) part. The "Trace Calculation" (2.3) is not done. This is 
      useful to request and use data from external devices in addition to other quantities or do some calculation.

  This function is optional, but if it is implemented, also the functions "RsVnaGetUnit", "RsVnaGetNumberOfHumanNames" and "RsVnaCalculateTrace" are required.

  @param[in]     taskId             Contains the task ID to identify the corresponding task type.
  @param[in,out] inputType          Contains the input type.
  @param[in]     numberOfHumanNames Contains the number of human names as provided by the method "RsVnaGetNumberOfHumanNames()".
  @param[in,out] humanNamesList     Contains a list of human names (same as in SCPI), required for the trace calculation later.
  @return        0 if no error occurred, else an error code to identify the occurred error.
                 A detailed error information is provided by the method "RsVnaGetErrorInfo()".
  */
  RS_VNA_API int RsVnaGetTraceRequirements(const struct RsVnaTaskId taskId,
                                           enum ERsVnaInputType* inputType,
                                           const unsigned int numberOfHumanNames,
                                           char** humanNamesList);

  /** Calculates the trace.

  This function should provide the data used for the trace identified by "RsVnaTaskId". The function is called for each sweep point of the trace. The input
  data is provided as requested by "RsVnaGetNumberOfHumanNames" and "RsVnaGetTraceRequirements". The output is the complex trace value for the specified point.
  (If real values are desired set the imaginary part to zero.)

  This function is optional, but if it is implemented, also the functions "RsVnaGetUnit", "RsVnaGetNumberOfHumanNames" and "RsVnaGetTraceRequirements"
  are required.

  @param[in]     taskId              Contains the task ID to identify the corresponding task type.
  @param[in]     position            Contains the current measurement point within the sweep (1-based).
  @param[in]     numberOfSweepPoints Contains the number of measurement points.
  @param[in]     numberOfTraceData   Contains the number of trace data.
  @param[in]     traceDataList       Contains a list of trace data for the current measurement point.
  @param[in,out] outputValue         Contains the calculated complex value for the current measurement point.
  @return        0 if no error occurred, else an error code to identify the occurred error.
                 A detailed error information is provided by the method "RsVnaGetErrorInfo()".
  */
  RS_VNA_API int RsVnaCalculateTrace(const struct RsVnaTaskId taskId,
                                     const unsigned int position,
                                     const unsigned int numberOfSweepPoints,
                                     const unsigned int numberOfTraceData,
                                     const struct RsVnaTraceData** traceDataList,
                                     struct RsVnaComplex* outputValue);

  /** Gets the interrupt requests.

  For the given "RsVnaTaskId" here the preparations for the later calls to "RsVnaInterruptCallback" are done. The firmware must know at which point during the
  measurements a call of "RsVnaInterruptCallback" should occur. At the defined points the firmware stops the measurement sequence and calls 
  "RsVnaInterruptCallback". After the function returned the sequence proceeds. There are four options. These are identical with the locations for Trigger 
  Signals specified in the manual. Some examples for a channel with drive port 1 and drive port 2 and 7 sweep points:
  \code
  Drive port:       1   1   1   1   1   1   1   2   2   2   2   2   2   2 
  Position:         1   2   3   4   5   6   7   1   2   3   4   5   6   7  
  Channel Both:    x|                       |   |                       |x
  Drive Port Both: x|                       |x  |                       |    // for the selection of port 1 as driving
  Drive Port Both:  |                       |  x|                       |x   // for the selection of port 2 as driving
  Drive Port Both: x|                       |x x|                       |x   // for a permanent selection
  Meas Point Both: x|x x x x x x x x x x x x|x  |                       |    // for the selection of port 1 as driving
  Meas Point Both:  |                       |  x|x x x x x x x x x x x x|x   // for the selection of port 2 as driving
  Meas Point Both: x|x x x x x x x x x x x x|x x|x x x x x x x x x x x x|x   // for a permanent selection
  \endcode
  If no interrupt is specified the function "RsVnaInterruptCallback" is not called.

  This function is optional, but if it is implemented and an interrupt is specified, also the function "RsVnaInterruptCallback" is required.

  @param[in]     taskId                          Contains the task ID to identify the corresponding task type.
  @param[in,out] interruptPositionForChannel     Contains the interrupt position, which describes where the interrupt should be occurred,
                                                 for the interrupt type "Channel".
  @param[in,out] interruptPositionForSegment     Contains the interrupt position, which describes where the interrupt should be occurred,
                                                 for the interrupt type "Segment".
  @param[in,out] interruptPositionForDrivingPort Contains the interrupt position, which describes where the interrupt should be occurred,
                                                 for the interrupt type "Driving Port".
  @param[in,out] interruptPositionForMeasPoint   Contains the interrupt position, which describes where the interrupt should be occurred,
                                                 for the interrupt type "Measurement Point".
  @return        0 if no error occurred, else an error code to identify the occurred error.
                 A detailed error information is provided by the method "RsVnaGetErrorInfo()".
  */
  RS_VNA_API int RsVnaGetInterruptRequests(const struct RsVnaTaskId taskId,
                                           enum ERsVnaInterruptPosition* interruptPositionForChannel,
                                           enum ERsVnaInterruptPosition* interruptPositionForSegment,
                                           enum ERsVnaInterruptPosition* interruptPositionForDrivingPort,
                                           enum ERsVnaInterruptPosition* interruptPositionForMeasPoint);

  /** Calculates the interrupt callback.

  This function is called at the positions defined by "RsVnaGetInterruptRequests" for the given "RsVnaTaskId". The call is designed to control additional 
  devices. This can be used to control the devices or read data. There are two types of callbacks:
  - for permanent usage: RsVnaTaskId::drivingPort is zero
    In this case no return values are expected. "outputValueList" and "positionOfLastOutputValue" cannot be accessed.
  - for usage when one port is driving: RsVnaTaskId::drivingPort is unequal zero
    In this case return values are expected. The parameter "positionOfLastOutputValue" allows the DLL to control the point up to which the data in 
    "outputValueList" is inserted. The firmware will only use the values in "outputValueList" up to the position defined by "outputValueList". The 
    "positionOfLastOutputValue" is assumed to be equal to the number of sweep points with the last call in the channel to avoid a blocking.

  This function is optional, but if it is implemented, also the function "RsVnaGetInterruptRequests" is required.

  @param[in]     taskId                    Contains the task ID to identify the corresponding task type.
  @param[in]     deviceAddress             Contains the device address of an external device.
  @param[in]     position                  Contains the current measurement point within the sweep (1-based).
                                           The position can also be 0 in case of: interrupt type "Channel" and interrupt "PRE" or "BOTH".
                                           The position can also be (numberOfSweepPoints + 1) in case of: interrupt type "Channel" and interrupt "POST" or "BOTH".
  @param[in]     numberOfSweepPoints       Contains the number of measurement points.
  @param[in]     interruptType             Contains the interrupt type, which describes how often an interrupt should be occurred.
  @param[in]     interruptPosition         Contains the interrupt position, which describes where the interrupt should be occurred.
  @param[in,out] outputValueList           Contains a list of calculated complex values. The number of calculated complex values corresponds to
                                           the number of measurement points which are relevant for the current interrupt callback. This pointer has 
                                           numberOfSweepPoints entries (In case of permanent usage this is zero - do not use).
  @param[in,out] positionOfLastOutputValue Contains the measurement point of the last calculated complex value.
  @return        0 if no error occurred, else an error code to identify the occurred error.
                 A detailed error information is provided by the method "RsVnaGetErrorInfo()".
  */
  RS_VNA_API int RsVnaInterruptCallback(const struct RsVnaTaskId taskId,
                                        const char* deviceAddress,
                                        const unsigned int position,
                                        const unsigned int numberOfSweepPoints,
                                        const enum ERsVnaInterruptType interruptType,
                                        const enum ERsVnaInterruptPosition interruptPosition,
                                        struct RsVnaComplex* outputValueList,
                                        unsigned int* positionOfLastOutputValue);

  /** Gets detailed information of a specific error code.

  This function is called always if any of the other functions returns a non zero return value. The return value of this function is passed as parameter here.
  The function shall return an error text which describes the reason for the occurred error. The message is displayed in the firmware.

  This function is optional, if it is not implemented only the error code instead of an error text is displayed.

  @param[in]     errorCode Contains a specific error code of which the information is requested.
  @param[in,out] errorText Contains the error text corresponding to the passed error code.
  @return        0 if no error occurred, else no detailed error information could be received. 
                 (Not zero return values do not trigger a call to this function).
  */
  RS_VNA_API int RsVnaGetErrorInfo(const int errorCode, char** errorText);

#ifdef __cplusplus
}
#endif

#endif /*RS_VNA_CUSTOM_DLL_INTERFACE_28F7221D_FAA2_450B_B88A_BB6BED9C0262*/
